#include <vector>
#include <ctime>
#include <iostream>
#include <utility>

using indexAndValue = std::pair<unsigned int, int>;
using twoDimensionalIntVector = std::vector< std::vector<int> >;
using indexInfoVector = std::vector<indexAndValue>;

void fillByRandomNumbers(twoDimensionalIntVector& twoDimIntVec)
{
   const unsigned int xSize(1 + rand() % 10);
   const unsigned int ySize(1 + rand() % 10);

   twoDimIntVec.reserve(xSize);
   twoDimIntVec.resize(xSize);

   for (int i = 0; i < xSize; ++i)
   {
      twoDimIntVec[i].reserve(ySize);
      twoDimIntVec[i].resize(ySize);

      for (int j = 0; j < ySize; ++j)
      {
         twoDimIntVec[i][j] = 0 + rand() % 20 - 10 ;
      }
   }
}

void show(const twoDimensionalIntVector& twoDimIntVec)
{
   const int xSize(twoDimIntVec.size());
   const int ySize(twoDimIntVec.front().size());

   std::cout << xSize << "x" << ySize << std::endl << std::endl;

   for (int j = ySize - 1; j > -1; --j)
   {
      for (int i = 0; i < xSize; ++i)
      {
         std::cout << twoDimIntVec[i][j];

         if (j < ySize)
         {
            std::cout << "\t";
         }
         else
         {
            std::cout << std::endl;
         }
      }

      std::cout << std::endl;
   }
}

indexInfoVector getIndexOfColumnsWithMinElement(const twoDimensionalIntVector& twoDimIntVec)
{
   indexInfoVector indexTable;

   auto getMinElement = [&twoDimIntVec](const unsigned int& index) -> const int
   {
      int minElement;

      if (index < twoDimIntVec.size())
      {
         const std::vector<int>& columnRef = twoDimIntVec[index];
         minElement = columnRef.front();

         for (const auto& currentElement : columnRef)
         {
            if (minElement > currentElement)
            {
               minElement = currentElement;
            }
         }
      }

      return minElement;
   };

   const int twoDimIntVecSize(twoDimIntVec.size());
   const int firstArrayIndex(0);

   if (twoDimIntVecSize == firstArrayIndex)
   {
      int minElement(getMinElement(firstArrayIndex));
      indexTable.push_back(indexAndValue(firstArrayIndex, minElement));
   }
   else if (twoDimIntVecSize > firstArrayIndex)
   {
      int currentMinElement;
      int minElement(getMinElement(firstArrayIndex));
      indexTable.push_back(indexAndValue(firstArrayIndex, minElement));

      for (int i = 1; i < twoDimIntVecSize; ++i)
      {
         currentMinElement = getMinElement(i);

         if (minElement > currentMinElement)
         {
            indexTable.clear();
            minElement = currentMinElement;            
            indexTable.push_back(indexAndValue(i, minElement));
         }
         else if (minElement == currentMinElement)
         {
            indexTable.push_back(indexAndValue(i, minElement));
         }
      }
   }

   indexTable.shrink_to_fit();

   return indexTable;
}

double getAverageValueOfPositiveElementsFromColumn(const twoDimensionalIntVector& twoDimIntVec, const int& index)
{
   double avarageValue(0);
   const int twoDimIntVecSize(twoDimIntVec.size());

   if (index < twoDimIntVecSize)
   {
      int valueSumm(0);
      int valueCount(0);
      const auto& columnRef(twoDimIntVec[index]);

      for (const int& currentValue : columnRef)
      {
         if (0 <= currentValue)
         {
            valueSumm += currentValue;
            ++valueCount;
         }
      }

      if (0 < valueCount)
      {
         avarageValue = valueSumm / static_cast<double>(valueCount);
      }
   }

   return avarageValue;
}

int main()
{
   srand(clock());

   twoDimensionalIntVector twoDimVec;
   fillByRandomNumbers(twoDimVec);

   auto indexInfoCollection = getIndexOfColumnsWithMinElement(twoDimVec);

   show(twoDimVec);
   std::cout << std::endl;

   const int indexShift(1);
   int index;

   for (auto& columnMinimalValuePair : indexInfoCollection)
   {
      index = columnMinimalValuePair.first;

      std::cout << "Column Index: " << index + indexShift << std::endl;
      std::cout << "Minimal Value: " << columnMinimalValuePair.second << std::endl;
      std::cout << "Average Value: " << getAverageValueOfPositiveElementsFromColumn(twoDimVec, index) << std::endl;

      std::cout << std::endl;
   }

   return 0;
}

